/*
 * Created on 24 aot 2009
 */
package deltadore.x2d.example;

import gnu.io.*;
import deltadore.x2d.control.*;
import deltadore.x2d.msg.*;
import deltadore.x2d.msg.actuator.*;
import deltadore.x2d.msg.metering.*;
import deltadore.x2d.msg.sensor.*;

/**
 * @author y-lebret
 */
public class X2dExampleProgram implements MessageReceivedListener {
	private Console console;
	
	public X2dExampleProgram() throws Exception {
		console = new Console("X2D Example Program");
				
		console.write("Port choice (default COM1) : ");
		String portName = console.readLine();
		if (portName.equals("")) { portName = "COM1"; }
		
		CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(portName);
		SerialPort port = (SerialPort)portId.open("X2dExampleProgram", 2000);
		port.setSerialPortParams(19200,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);
		port.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		
		X2dController controller = new X2dController();
		controller.addMessageReceivedListener(this);
		controller.open(port.getInputStream(), port.getOutputStream());
		
		console.writeLine();
		console.writeLine("Menu");
		console.writeLine("=====================================");
		console.writeLine("Q => quit");
		console.writeLine("0 => enrollment mode");
		console.writeLine("1 => enroll");
		console.writeLine("3 => command");
		console.writeLine("4 => request variation 0%");
		console.writeLine("5 => request variation 25%");
		console.writeLine("6 => request variation 100%");
		console.writeLine("7 => request meter reading");
		console.writeLine();
		
		while(true) {
			char menuChoice = console.readKey();
			if (menuChoice == 'Q' || menuChoice == 'q') { break; }
			else if (menuChoice == '0') {
				controller.setEnrollmentListeningMode(true);
			}
			else if (menuChoice == '1') {
				EnrollmentMessage msg = (EnrollmentMessage)controller.createMessage(EnrollmentMessage.class);
				controller.sendMessage(msg);
			}
			else if (menuChoice == '3') {
				BasicCommandMessage msg = (BasicCommandMessage)controller.createMessage(BasicCommandMessage.class);
				controller.sendMessage(msg);
			}
			else if (menuChoice == '4') {
				VariationMessage msg = (VariationMessage)controller.createMessage(VariationMessage.class);
				msg.setValue(0);
				controller.sendMessage(msg);
			}
			else if (menuChoice == '5') {
				VariationMessage msg = (VariationMessage)controller.createMessage(VariationMessage.class);
				msg.setValue(25);
				controller.sendMessage(msg);
			}
			else if (menuChoice == '6') {
				VariationMessage msg = (VariationMessage)controller.createMessage(VariationMessage.class);
				msg.setValue(100);
				controller.sendMessage(msg);
			}
			else if (menuChoice == '7') {
				BasicMeterReadingMessage msg = (BasicMeterReadingMessage)controller.createMessage(BasicMeterReadingMessage.class);
				msg.setAnswerRequest(true);
				controller.sendMessage(msg);
			}
		}
		
		controller.close();
		port.close();
		
		System.exit(0);
	}
	
	public static void main(String[] args) throws Exception {
		new X2dExampleProgram();
	}
	
	public void messageReceived(MessageReceivedEvent evt) {
		Message msg = evt.getMessage();
		if (msg instanceof DetectionMessage) {
			DetectionMessage detectionMsg = (DetectionMessage)msg;
			if (detectionMsg.getDetectorType() == DetectorType.PerimetricContact){
				if (detectionMsg.isAlarmTriggered()) {
					console.writeLine("Opening detected");
				}
				else {
					console.writeLine("Closing detected");				
				}				
			}
			else if (detectionMsg.getDetectorType() == DetectorType.VolumetricInfrared){
				if (detectionMsg.isAlarmTriggered()) {
					console.writeLine("Presence detected");
				}
				else {
					console.writeLine("Presence undetected");				
				}								
			}
		}
		else if (msg instanceof BasicMeterReadingMessage) {
			BasicMeterReadingMessage readingMsg = (BasicMeterReadingMessage)msg;
			console.write("Heating=" + readingMsg.getHeatingRegister());
			console.write(" HotWater=" + readingMsg.getHotWaterRegister());
			console.writeLine(" Total=" + readingMsg.getTotalRegister());
		}
		else if (msg instanceof ExternalTemperatureMessage){
			ExternalTemperatureMessage externalTempMsg = (ExternalTemperatureMessage)msg;
			console.writeLine("External temperature is " + externalTempMsg.getTemperature() + " C");
		}
		else if (msg instanceof InternalTemperatureMessage){
			InternalTemperatureMessage internalTempMsg = (InternalTemperatureMessage)msg;
			console.writeLine("Internal temperature is " + internalTempMsg.getTemperature() + " C");			
		}
	}
}